﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace JeuAvecDesKilometres
{
    public class Botte : Carte
    {
        public Botte(string CheminImage, TypeEmbuche LeType)
            : base(CheminImage, 100)
        {
            EmbucheImmunisee = LeType;
        }

        public TypeEmbuche EmbucheImmunisee { get; private set; }

        /// <summary>
        /// Réécriture de ToString pour afficher le nom de la botte
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            switch (EmbucheImmunisee)
            {
                case TypeEmbuche.Accident:
                    return "As du volant";

                case TypeEmbuche.Crevaison:
                    return "Roue increvable";

                case TypeEmbuche.FeuRouge | TypeEmbuche.LimitationVitesse:
                    return "Camion de pompier";

                case TypeEmbuche.PanneEssence:
                    return "Citerne d'essence";
            }

            //on est obligé de mettre un default ou aprés le swicth return quelque chose car ToString retourne un résultat, et même si on a testé toutes les options de TypeEmbuche, le compilateur considère qu'un cas non prévu pourrait se présenter et qu'il n'y aurait alors pas de résultat
            return "Erreur";
        }

        /// <summary>
        /// Cette méthode est appelée par le joueur en cas de coup fourré
        /// </summary>
        public void CoupFourreParLeJoueur()
        {
            this.Points += 300;
        }

        /// <summary>
        /// Méthode de classe qui génère la liste de Bottes
        /// </summary>
        /// <returns></returns>
        public static List<Carte> Generer()
        {
            List<Carte> resultat = new List<Carte>();

            resultat.Add(new Botte("Image du camion de pompier", TypeEmbuche.FeuRouge | TypeEmbuche.LimitationVitesse));
            resultat.Add(new Botte("Image de la citerne d'essence", TypeEmbuche.PanneEssence));
            resultat.Add(new Botte("Image de la roue increvable", TypeEmbuche.Crevaison));
            resultat.Add(new Botte("Image de l'as du volant", TypeEmbuche.Accident));

            return resultat;
        }

        /// <summary>
        /// Implémentation de ordre dans Botte
        /// </summary>
        public override int Ordre
        {
            get 
            { 
                switch(EmbucheImmunisee)
                {
                    case TypeEmbuche.Accident://As du Volant
                        return 0;

                    case TypeEmbuche.FeuRouge | TypeEmbuche.LimitationVitesse://Camion de Pompier
                        return 1;

                    case TypeEmbuche.PanneEssence://Citerne d'essence
                        return 2;

                    default: //il ne reste que la roue de secours
                        return 3;
                }
            }
        }
    }
}
