﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace JeuAvecDesKilometres
{
    public static class Moteur
    {
        public static event EvenementCarte CarteInterdite; //évènement signalant que le joueur tente de poser une carte à l'encontre des règles. 

        public delegate void EvenementJoueur(Joueur LeJoueur);//signature de l'évènement
        public static event EvenementJoueur ProchainJoueur;//évènement annonçant le joueur suivant
        public static event EvenementJoueur PartieGagnee;//évènement annonçant qui a gagné
        public static event EvenementJoueur JoueurDisqualifie;//évènement annonçant qui est disqualifié
        public static event EvenementJoueur PiocheDabord;//évènement annonçant que le joueur doit piocher d'abord

        public delegate void EvenementMoteurJeu();//signature de l'évènement
        public static event EvenementMoteurJeu NouvellePioche;//évènement annonçant la nouvelle pioche
        public static event EvenementMoteurJeu CoupFourre;//évènement annonçant un coup fourré

        /// <summary>
        /// Distance à parcourir pour terminer une partie
        /// </summary>
        public static int DistanceAParcourir { get; private set; }

        /// <summary>
        /// Liste des joueurs
        /// </summary>
        public static List<Joueur> Joueurs { get; private set; }

        /// <summary>
        /// Collection de cartes défaussées
        /// </summary>
        public static ObservableCollection<Carte> Defausse { get; set; }

        public static ObservableCollection<Carte> Pioche { get; set; }
        /// <summary>
        /// Collection de cartes à piocher
        /// </summary>

        /// <summary>
        /// Joueur dont c'est le tour de jouer
        /// </summary>
        public static Joueur JoueurEnCours { get { return leJoueurEnCours; } }

        private static Joueur leJoueurEnCours;

        /// <summary>
        /// Indique si une partie est en cours
        /// </summary>
        public static bool PartieEnCours { get; private set; }

        /// <summary>
        /// Initialise les éléments de jeu avant la première partie
        /// </summary>
        /// <param name="NomsJoueurs">Liste avec les noms des joueurs</param>
        public static void InitPremierePartie(List<string> NomsJoueurs)
        {
            Dictionary<string, Difficulte> lesJoueurs = new Dictionary<string, Difficulte> ();
            foreach (string joueur in NomsJoueurs)
            {
                lesJoueurs.Add(joueur,Difficulte.Facile);
            }

            InitPremierePartie(lesJoueurs);
        }

        /// <summary>
        /// Initialise les éléments de jeu avant la première partie
        /// </summary>
        /// <param name="LesJoueurs">Dictionnaire avec les noms des joueurs et leurs niveau</param>
        public static void InitPremierePartie(Dictionary<string, Difficulte> LesJoueurs)
        {
            Defausse = new ObservableCollection<Carte>();
            Pioche = new ObservableCollection<Carte>();
            Pioche.CollectionChanged += Pioche_CollectionChanged;
            Joueurs = new List<Joueur>();

            //Crée les joueurs et s'abonne aux évènements
            foreach (var joueur in LesJoueurs)
            {
                Joueur j = new Joueur(joueur.Key,joueur.Value);
                j.CoupFourre += j_CoupFourre;
                j.DistanceDepassee += j_DistanceDepassee;
                j.JaiGagne += j_JaiGagne;
                j.FiniMonTour += j_FiniMonTour;
                j.CarteInterdite += j_CarteInterdite;
                j.PiocherDabord += j_PiocherDabord;

                Joueurs.Add(j);
            }


            DistanceAParcourir = Joueurs.Count > 3 ? 700 : 1000;//La distance à parcourir est 700 à partir de 4 joueurs, sinon 1000

            NouvellePartie();

        }

        /// <summary>
        /// Annonce que le joueur doit piocher avant de joueur
        /// </summary>
        /// <param name="LeJoueur"></param>
        static void j_PiocherDabord(Joueur LeJoueur)
        {
            if (Moteur.PiocheDabord != null)
                PiocheDabord(LeJoueur);

        }

        /// <summary>
        /// Transmet l'alerte qu'un joueur tente de joueur une carte "illégale"
        /// </summary>
        /// <param name="LeJoueur"></param>
        /// <param name="LaCarte"></param>
        static void j_CarteInterdite(Joueur LeJoueur, Carte LaCarte, Interdictions Raison)
        {
            if (Raison != Interdictions.Patienter)//Patienter est dans le cas ou un joueur "A" de niveau "Normal" peut jouer un coup fourré et qu'un autre joueur "B" joue avant les 10 secondes, le joueur "B" ne doit pas être pénalisé
            {

                switch (LeJoueur.Niveau)//selon le niveau du joueur, on défausse la carte, ce qui passe au joueur suivant
                {
                    case Difficulte.Difficile:
                    case Difficulte.Expert:
                        Raison = Interdictions.Inconnue;//à ces niveaux, la raison n'est plus affichée.
                        LeJoueur.Defausse(LaCarte, Raison != Interdictions.PasSonTour);
                        break;

                    default:
                        if (Raison == Interdictions.PiocheInterdite)
                            LeJoueur.Defausse(LaCarte, false);
                        break;

                }
            }

            if (Moteur.CarteInterdite != null)
                Moteur.CarteInterdite(LeJoueur, LaCarte, Raison);
        }

        /// <summary>
        /// Quand un joueur à finit, on passe au suivant
        /// </summary>
        /// <param name="LeJoueur"></param>
        static void j_FiniMonTour(Joueur LeJoueur)
        {
            JoueurSuivant(LeJoueur);
        }

        /// <summary>
        /// Signale la fin de la partie
        /// </summary>
        /// <param name="LeJoueur"></param>
        /// <param name="Kilometres"></param>
        private static void j_JaiGagne(Joueur LeJoueur, int Kilometres)
        {
            PartieEnCours = false;

            foreach (Joueur j in Joueurs)
                j.FinPartie(numeroPartie);

            if (Moteur.PartieGagnee != null)
                Moteur.PartieGagnee(LeJoueur);

        }


        private static int numeroPartie = 0;

        /// <summary>
        /// Démarre une nouvelle partie
        /// </summary>
        public static void NouvellePartie()
        {
            numeroPartie++;

            //génération des cartes
            List<Carte> toutesLesCartes = Kilometre.Generer();
            toutesLesCartes.AddRange(Embuche.Generer());
            toutesLesCartes.AddRange(Parade.Generer());
            toutesLesCartes.AddRange(Botte.Generer());

            //Efface les cartes présentes dans le jeu
            Pioche.Clear();
            Defausse.Clear();
            foreach (Joueur j in Joueurs)
                j.NouvellePartie();

            //mélange des cartes
            Melanger(toutesLesCartes);

            //distribution des cartes 6 par joueur, une par une chacun son tour
            for (int i = 0; i < 6; i++)
            {
                for (int j = 0; j < Moteur.Joueurs.Count; j++)
                {
                    Moteur.Joueurs[j].Pioche();
                }
            }

            leJoueurEnCours = Joueurs[0];
            AnnonceProchainJoueur();

            PartieEnCours = true;

        }

        private static List<int> indexJoueursDisqualifies = new List<int>();
        /// <summary>
        /// Un joueur a dépassé la distance=> il est éliminé
        /// </summary>
        /// <param name="LeJoueur"></param>
        /// <param name="Kilometres"></param>
        static void j_DistanceDepassee(Joueur LeJoueur, int Kilometres)
        {
            if (LeJoueur.Niveau == Difficulte.Expert)//si le joueur est expert, cette erreur le disqualifie
            {
                indexJoueursDisqualifies.Add(Joueurs.IndexOf(LeJoueur));
                if (Moteur.JoueurDisqualifie != null)
                    Moteur.JoueurDisqualifie(LeJoueur);
            }
        }


        /// <summary>
        /// Change le joueur après un coup fourré
        /// </summary>
        /// <param name="LeJoueur"></param>
        private static void j_CoupFourre(Joueur LeJoueur)
        {
            leJoueurEnCours = LeJoueur;
            AnnonceProchainJoueur();

            if (Moteur.CoupFourre != null)
                Moteur.CoupFourre();
        }

        /// <summary>
        /// Annonce le joueur suivant
        /// </summary>
        private static void AnnonceProchainJoueur()
        {

            foreach(Joueur j in Joueurs.Where(j => j.Actif))
                j.Actif = false;//"desactive" l'ancien joueur

            JoueurEnCours.Actif = true;//"active" le joueur en cours

            JoueurEnCours.PiocheAutomatique();//fait piocher le joueur suivant
            if (ProchainJoueur != null)
                ProchainJoueur(JoueurEnCours);
        }

        /// <summary>
        /// Surveille la pioche, et la refait à partir de la défausse quand elle est vide
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private static void Pioche_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.Action != System.Collections.Specialized.NotifyCollectionChangedAction.Remove)
                return;

            if (Pioche.Count == 0)
            {
                Melanger(Defausse.ToList());
                if (Moteur.NouvellePioche != null)
                    Moteur.NouvellePioche();
            }
        }

        /// <summary>
        /// Mélange les cartes et (re)crée la pioche
        /// </summary>
        /// <param name="Cartes"></param>
        private static void Melanger(List<Carte> Cartes)
        {
            Random rnd = new Random();
            do
            {
                int index = rnd.Next(Cartes.Count);//génère un entier aléatoire compris entre 0 et Cartes.Count -1
                Pioche.Add(Cartes[index]);//ajoute la carte à cet index dans la pioche
                Cartes.RemoveAt(index);//enlève la carte de la collection d'origine
            } while (Cartes.Count > 0);
        }

        /// <summary>
        /// Passe au joueur suivant
        /// </summary>
        private static void JoueurSuivant(Joueur LeJoueur)
        {
            int indexJoueur = Joueurs.IndexOf(LeJoueur);

            if (indexJoueur == Joueurs.Count - 1)
                indexJoueur = 0;
            else
                indexJoueur++;

            leJoueurEnCours = Joueurs[indexJoueur];

            if (indexJoueursDisqualifies.Contains(indexJoueur))//si le joueur est disqualifié on passe au suivant de manière récursive
                JoueurSuivant(LeJoueur);

            else AnnonceProchainJoueur();
        }
    }

}
