﻿using JeuAvecDesKilometres;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Tuto_Objets_Csharp_WPF
{
    /// <summary>
    /// Logique d'interaction pour MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            Dictionary<string, Difficulte> nomsJoueurs = new Dictionary<string, Difficulte>();
            nomsJoueurs.Add("Joueur A", Difficulte.Normale);
            nomsJoueurs.Add("Joueur B", Difficulte.Normale);
            nomsJoueurs.Add("Joueur C", Difficulte.Normale);

            Moteur.InitPremierePartie(nomsJoueurs);

            lesJoueurs.DataContext = Moteur.Joueurs;//binde tous les joueurs
            lblPioche.DataContext = Moteur.Pioche.Count;//binde le nombre de cartes dans la pioche
            lstDefausse.DataContext = Moteur.Defausse;//binde la defausse

            Moteur.PartieGagnee += Moteur_PartieGagnee;
            Moteur.CarteInterdite += Moteur_CarteInterdite;
            Moteur.CoupFourre += Moteur_CoupFourre;
            Moteur.JoueurDisqualifie += Moteur_JoueurDisqualifie;
            Moteur.PiocheDabord += Moteur_PiocheDabord;

            
        }

        private void butPiocher_Click(object sender, RoutedEventArgs e)
        {
            Joueur j = TrouveJoueur(sender);
            j.Pioche();
        }

        private void butJouer_Click(object sender, RoutedEventArgs e)
        {
            Joueur j = TrouveJoueur(sender);
            Carte laCarte = TrouveCarte(sender);

            if (laCarte == null)
                return;

            if (laCarte is Botte)
                j.Jouer((Botte)laCarte);
            else if (laCarte is Kilometre)
                j.Jouer((Kilometre)laCarte);
            else if (laCarte is Parade)
                j.Jouer((Parade)laCarte);
            else
            {
                attaquant = j;
                attaque = (Embuche)laCarte;
                AfficheMessage("Cliquer sur la cible");
            }
        }
        private Joueur attaquant;
        private Embuche attaque;

        private void butDefausser_Click(object sender, RoutedEventArgs e)
        {
            Joueur j = TrouveJoueur(sender);
            Carte laCarte = TrouveCarte(sender);
            if (laCarte == null)
                return;

            j.Defausse(laCarte);
        }


        private Joueur TrouveJoueur(object sender)
        {
            //On peut remonter à l'instance qui a bindé le bouton clické
            Button bouton = (Button)sender;
            object instance = bouton.DataContext;
            return (Joueur)instance;
        }

        private Carte TrouveCarte(object sender)
        {
            //on peut aussi remonter à la carte sélectionnée dans la liste
            Button bouton = (Button)sender;
            StackPanel parent1 = (StackPanel)bouton.Parent;
            
            StackPanel parent2;
            if (parent1.Name == "splJoueur")
                parent2 = parent1;
            else
                parent2 = (StackPanel)parent1.Parent;

            ListBox main = parent2.Children.OfType<ListBox>().Single(lb => lb.Name == "Main");
            return (Carte)main.SelectedItem;

        }

        private void lesJoueurs_MouseUp(object sender, MouseButtonEventArgs e)
        {
            if (attaquant != null && lesJoueurs.SelectedItem != attaquant)
            {
                attaquant.Jouer(attaque, (Joueur)lesJoueurs.SelectedItem);
                attaque = null;
                attaquant = null;
            }
            lesJoueurs.SelectedIndex = -1;
        }


        void Moteur_PiocheDabord(Joueur LeJoueur)
        {
            AfficheMessage("Le joueur doit piocher");

        }

        private void AfficheMessage(string Texte)
        {
            DoubleAnimation animation = new DoubleAnimation(1, 0, TimeSpan.FromSeconds(3));
            lblMessage.BeginAnimation(Rectangle.OpacityProperty, animation);

            lblMessage.Content = Texte;

        }

        void Moteur_JoueurDisqualifie(Joueur LeJoueur)
        {
            AfficheMessage(LeJoueur.Nom + " est disqualifié");
        }

        void Moteur_CoupFourre()
        {
            AfficheMessage("Coup fourré!");
        }

        void Moteur_CarteInterdite(Joueur LeJoueur, Carte LaCarte, Interdictions Raison)
        {
            string[] messages = {"Joueur immunisé","Limitation de vitesse","Embûche à parer","2 cartes 200 km maximum","Parade inutile",
                                "Parez l'embûche avant de poser le feu vert","La cible est déjà attaquée","Joueur déjà bridé",
                                "Carte interdite pour une raison inconnue","Distance dépassée","Veuillez patienter","Ce n'est pas votre tour","Pioche interdite"};
            AfficheMessage(messages[(int)Raison]);
        }

        void Moteur_PartieGagnee(Joueur LeJoueur)
        {
            Joueur lautre = Moteur.Joueurs.Single(j => j != LeJoueur);
            StringBuilder str = new StringBuilder(LeJoueur.Nom);
            str.AppendLine(" a gagné!");
            str.AppendLine(string.Format("{0} marque {1} points.", LeJoueur.Nom, LeJoueur.Points));
            str.AppendLine(string.Format("{0} marque {1} points.", lautre.Nom, lautre.Points));


            MessageBox.Show(str.ToString());

        }
    }
}
