﻿Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.Text
Imports System.Threading.Tasks

Namespace JeuAvecDesKilometres
    Public Delegate Sub EvenementCarte(ByVal LeJoueur As Joueur, ByVal LaCarte As Carte, ByVal Raison As Interdictions) 'signature de l'évènement, je déclare ce délégué en dehors de la classe, ainsi, il est visible dans tout l'espace de nom

    Public Class Joueur
        Implements INotifyPropertyChanged
        ''' <summary>
        ''' Chronomètre pour le temps de jeu d'un coup fourré à partir du niveau "normal"
        ''' </summary>
        ''' <remarks>Ce chrono est une variable de classe, donc n'importe quel joueur peut le déclencher, et n'importe joueur peut le stopper</remarks>
        Private Shared chrono As New Stopwatch()

        ''' <summary>
        ''' Quand un joueur d'un niveau à partir du niveau "normal" a la possibilité de jouer un coup fourré, il est stocké dans cette varible static
        ''' </summary>
        Private Shared coupFourrePossible As Joueur

        Public Event CarteInterdite As EvenementCarte 'évènement signalant que le joueur tente de poser une carte à l'encontre des règles.

        Public Delegate Sub EvenementKilometres(ByVal LeJoueur As Joueur, ByVal Kilometres As Integer) 'signature de l'évènement
        Public Event JaiGagne As EvenementKilometres 'évènement signalant que la distance est atteinte
        Public Event DistanceDepassee As EvenementKilometres 'évènement signalant que la distance est partagée

        Public Delegate Sub EvenementEmbuche(ByVal LeJoueur As Joueur) 'signature de l'évènement
        Public Event EmbucheRecue As EvenementEmbuche 'évènement signalant une embuche reçue
        Public Event CoupFourre As EvenementEmbuche 'évènement signalant un coup fourré
        Public Event FiniMonTour As EvenementEmbuche 'évènement signalant que le joueur à fini son tour.
        Public Event PiocherDabord As EvenementEmbuche 'évènement signalant que le joueur doit piocher avant de jouer.

        ''' <summary>
        ''' Constructeur 1
        ''' </summary>
        ''' <param name="LeNom">Nom du joueur</param>
        Public Sub New(ByVal LeNom As String) 'on appelle le constructeur plus complet avec une valeur par défaut
            Me.New(LeNom,Difficulte.Facile)
        End Sub

        ''' <summary>
        ''' Constructeur 2
        ''' </summary>
        ''' <param name="LeNom">Nom du joueur</param>
        ''' <param name="LeNiveau">et son niveau</param>
        Public Sub New(ByVal LeNom As String, ByVal LeNiveau As Difficulte)
            Nom = LeNom
            Main = New ObservableCollection(Of Carte)()
            Jeu = New ObservableCollection(Of Carte)() 'initialisation de la collection
            AddHandler Jeu.CollectionChanged, AddressOf Jeu_CollectionChanged 'Abonnement à l'évènement signalant un changement
            Niveau = LeNiveau
            Score = New Dictionary(Of Integer, Integer)()

            NouvellePartie()
        End Sub

        #Region "Propriétés"

'INSTANT VB NOTE: The variable nom was renamed since Visual Basic does not allow class members with the same name:
        Private nom_Renamed As String
        ''' <summary>
        ''' Nom du joueur
        ''' </summary>
        Public Property Nom As String
            Get
                Return nom_Renamed
            End Get
            Private Set(ByVal value As String)
                If nom_Renamed <> value Then
                    nom_Renamed = value
                    RaisePropertyChanged("Nom")
                End If
            End Set
        End Property

'INSTANT VB NOTE: The variable main was renamed since Visual Basic does not allow class members with the same name:
        Private main_Renamed As ObservableCollection(Of Carte)
        ''' <summary>
        ''' Collection de cartes dans la main du joueur
        ''' </summary>
        Public Property Main As ObservableCollection(Of Carte)
            Get
                Return main_Renamed
            End Get
            Set(ByVal value As ObservableCollection(Of Carte))
                If main_Renamed IsNot value Then
                    main_Renamed = value
                End If
            End Set
        End Property

        ''' <summary>
        ''' Collection de cartes representant le jeu du joueur
        ''' </summary>
        Public Property Jeu As ObservableCollection(Of Carte)

'INSTANT VB NOTE: The variable niveau was renamed since Visual Basic does not allow class members with the same name:
         Private niveau_Renamed As Difficulte
       ''' <summary>
        ''' Niveau de difficulté du joueur
        ''' </summary>
        Public Property Niveau As Difficulte
            Get
                Return niveau_Renamed
            End Get
            Set(ByVal value As Difficulte)
                If niveau_Renamed <> value Then
                    niveau_Renamed = value
                    RaisePropertyChanged("Niveau")
                End If
            End Set
        End Property

'INSTANT VB NOTE: The variable derniereCarte was renamed since Visual Basic does not allow class members with the same name:
        Private derniereCarte_Renamed As Carte
        ''' <summary>
        ''' Dernière carte posée dans le jeu, y compris les embuches
        ''' </summary>
        Public Property DerniereCarte As Carte
            Get
                Return derniereCarte_Renamed
            End Get
            Private Set(ByVal value As Carte)
                If derniereCarte_Renamed IsNot value Then
                    derniereCarte_Renamed = value
                    RaisePropertyChanged("DerniereCarte")
                End If
            End Set
        End Property

        ''' <summary>
        ''' Propriété en lecture seule, dont la valeur est le résultat d'un calcul
        ''' </summary>
        ''' <remarks>Etant en lecture seule, il faudra signaler le changement de valeur potentiel ailleurs dans cette classe</remarks>
        Public ReadOnly Property Points As Integer
            Get
                Dim lesPoints As Integer = 0
                For Each laCarte As Carte In Jeu
                    lesPoints += laCarte.Points 'grâce au polymorphisme, on peut additionner les points, peu importe le type de carte.
                Next laCarte


                Dim bonus As Integer = 0

                If Not Moteur.PartieEnCours Then
                    'à la fin de la partie si un (ou plusieurs) joueur n'a pas pu poser de kilomètre, les autres marquent 500 points de bonus
                    For Each leJoueur As Joueur In Moteur.Joueurs
                        If leJoueur Is Me Then
                            Continue For 'on passe à l'itération suivante
                        End If

                        If leJoueur.DistanceParcourue = 0 Then
                            bonus += 500 'on ajoute 500 au bonus pour se joueur sans distance
                        End If
                    Next leJoueur
                End If

                If laDistanceParcourue = 1000 Then 'bonus du gagnant
                    bonus += 400 '400 pour avoir gagné

                    If Jeu.OfType(Of Kilometre)().Count(Function(k) k.Distance = 200) = 0 Then 'pour la clause if, je filtre la collection Main, sur le type Kilometre, et je compte le nombre de Kilometre dont la distance vaut 0
                        bonus += 200 ' 200 de plus si aucune carte 200 utilisée
                    End If
                End If


                Return lesPoints + bonus
            End Get
        End Property

'INSTANT VB NOTE: The variable score was renamed since Visual Basic does not allow class members with the same name:
        Private score_Renamed As Dictionary(Of Integer,Integer)
        ''' <summary>
        ''' Dictionnaire stockant pour chaque partie les points réalisés
        ''' </summary>
        Public Property Score As Dictionary(Of Integer,Integer)
            Get
                Return score_Renamed
            End Get
            Private Set(ByVal value As Dictionary(Of Integer,Integer))
                If score_Renamed IsNot value Then
                    score_Renamed = value
                    RaisePropertyChanged("Score")
                End If
            End Set
        End Property

        Private laDistanceParcourue As Integer = 0
        ''' <summary>
        ''' Distance parcourrue pour cette partie
        ''' </summary>
        ''' <remarks>Cette propiété nécessite une variable interne car selon le niveau la distance est affichée ou pas, mais on doit connaitre la distance pour savoir si le joueur gagne ou dépasse
        ''' Etant en lecture seule, il faudra signaler le changement de valeur potentiel ailleurs dans cette classe</remarks>
        Public ReadOnly Property DistanceParcourue As Integer
            Get
                Select Case Niveau
                    Case Difficulte.Facile, Difficulte.Normale
                        Return laDistanceParcourue

                    Case Else
                        Return 0
                End Select
            End Get
        End Property

'INSTANT VB NOTE: The variable embucheEnCours was renamed since Visual Basic does not allow class members with the same name:
        Private embucheEnCours_Renamed As TypeEmbuche
        ''' <summary>
        ''' Embûches que subit actuellement le joueur
        ''' </summary>
        Public Property EmbucheEnCours As TypeEmbuche
            Get
                Return embucheEnCours_Renamed
            End Get
            Set(ByVal value As TypeEmbuche)
                If embucheEnCours_Renamed <> value Then
                    embucheEnCours_Renamed = value
                    RaisePropertyChanged("EmbucheEnCours")
                End If
            End Set
        End Property


'INSTANT VB NOTE: The variable immunite was renamed since Visual Basic does not allow class members with the same name:
        Private immunite_Renamed As TypeEmbuche
        ''' <summary>
        ''' Immunités dont dispose le joueur
        ''' </summary>
        Public Property Immunite As TypeEmbuche
            Get
                Return immunite_Renamed
            End Get
            Set(ByVal value As TypeEmbuche)
                If immunite_Renamed <> value Then
                    immunite_Renamed = value
                    RaisePropertyChanged("Immunite")
                End If
            End Set
        End Property

'INSTANT VB NOTE: The variable actif was renamed since Visual Basic does not allow class members with the same name:
        Private actif_Renamed As Boolean
        ''' <summary>
        ''' Retourne ou définit s'il s'agit du joueur dont c'est le tour
        ''' </summary>
        Public Property Actif As Boolean
            Get
                Return actif_Renamed
            End Get
            Set(ByVal value As Boolean)
                If actif_Renamed <> value Then
                    actif_Renamed = value
                    RaisePropertyChanged("Actif")
                End If
            End Set
        End Property

        #End Region

        #Region "Méthodes"
        ''' <summary>
        ''' (Re)initialise les variables
        ''' </summary>
        Public Sub NouvellePartie()
            EmbucheEnCours = TypeEmbuche.FeuRouge
            Immunite = TypeEmbuche.Aucune
            Main.Clear()
            Jeu.Clear()
        End Sub

        ''' <summary>
        ''' Signale au joueur que la partie est finie, il stocke ses points dans le dictionnaire.
        ''' </summary>
        ''' <param name="Numero"></param>
        Public Sub FinPartie(ByVal Numero As Integer)
            Score.Add(Numero, Points)
        End Sub

        ''' <summary>
        ''' Traitements à faire après qu'une carte ait été ajoutée au jeu
        ''' </summary>
        ''' <param name="sender"></param>
        ''' <param name="e"></param>
        Private Sub Jeu_CollectionChanged(ByVal sender As Object, ByVal e As System.Collections.Specialized.NotifyCollectionChangedEventArgs)
            If e.Action <> System.Collections.Specialized.NotifyCollectionChangedAction.Add Then
                Return
            End If

            coupFourrePossible = Nothing 'RAZ de la variable, si un joueur joue avant le coup fourré, celui qui pouvait perd cette opportunité

            DerniereCarte = Jeu(e.NewStartingIndex)

            If TypeOf DerniereCarte Is Embuche Then 'si la dernière carte recue est une embuche, car Jeu contient des Cartes
                RaiseEvent EmbucheRecue(Me) 'on émet l'évènement

                Dim lEmbuche As Embuche = CType(DerniereCarte, Embuche) 'pour travailler sur une Embuche, à partir d'une Carte, il faut "caster" (forcer la conversion) la Carte en Embuche

                'on regarde si le joueur dispose de la botte pour faire un coup fourré
                For Each laCarte As Carte In Main
                    If TypeOf laCarte Is Botte AndAlso (CType(laCarte, Botte)).EmbucheImmunisee.HasFlag(lEmbuche.Type) Then

                        If Niveau = Difficulte.Facile Then
                            Joue(CType(laCarte, Botte)) 'On joue la botte
                            FaitCoupFourre(CType(laCarte, Botte)) 'on joue le coup fourré
                            Return 'on sort de la méthode, ça n'est pas la peine de continuer à cherche ce que l'on a déjà trouvé
                        Else
                            'on laisse au joueur 10 secondes pour réagir, on lance donc le chrono
                            coupFourrePossible = Me
                            chrono.Start()
                        End If
                    End If
                Next laCarte

                EmbucheEnCours = EmbucheEnCours Or lEmbuche.Type 'on affecte l'embûche
                If lEmbuche.Type <> TypeEmbuche.LimitationVitesse AndAlso (Not Immunite.HasFlag(TypeEmbuche.FeuRouge)) Then
                    EmbucheEnCours = EmbucheEnCours Or TypeEmbuche.FeuRouge 'on affecte un feu rouge pour que le joueur soit obligé de mettre un feu vert
                End If

            ElseIf TypeOf DerniereCarte Is Kilometre Then 'si la dernière carte est un kilomètre, on calcule la distance totale.
                Dim km As Kilometre = CType(DerniereCarte, Kilometre)
                Dim distance As Integer = km + laDistanceParcourue

                If distance > Moteur.DistanceAParcourir Then
                    RaiseEvent CarteInterdite(Me, DerniereCarte, Interdictions.DistanceDepassee)

                    RaiseEvent DistanceDepassee(Me, distance)

                    Return
                End If

                laDistanceParcourue = distance 'on valide la distance
                If laDistanceParcourue = Moteur.DistanceAParcourir Then
                    RaiseEvent JaiGagne(Me, laDistanceParcourue) 'on émet l'évènement avec le paramètre correspondant à la signature.

                End If
                RaisePropertyChanged("DistanceParcourue") 'signale un changement de distance
            End If
            RaisePropertyChanged("Points") 'signale un éventuel changement de points
        End Sub

        ''' <summary>
        ''' Joue un coup fourré
        ''' </summary>
        ''' <param name="LaBotte"></param>
        Private Sub FaitCoupFourre(ByVal LaBotte As Botte)
            LaBotte.CoupFourreParLeJoueur() 'appelle la méthode qui va mettre le bonus de points
            Jeu.Remove(DerniereCarte) 'on enlève l'embuche du jeu, ce qui va regénérer l'évènement Jeu_CollectionChanged, d'ou le premier test
            Moteur.Defausse.Add(DerniereCarte) 'on jette l'embuche à la défausse

            'on signale que l'on joue le coup fourré
            RaiseEvent CoupFourre(Me)

        End Sub

        ''' <summary>
        ''' Méthode permettant de jouer une Carte dans son jeu
        ''' </summary>
        ''' <param name="LaCarte"></param>
        ''' <remarks>Si cette méthode était publique, on se jouerait une embuche à soi-même</remarks>
        Private Sub Joue(ByVal LaCarte As Carte)
            If Not AutorisationDeJouer(LaCarte) Then
                Return
            End If

            If TypeOf LaCarte Is Kilometre Then
                Select Case EmbucheEnCours
                    Case TypeEmbuche.Aucune 'on peut rouler

                    Case TypeEmbuche.LimitationVitesse 'il n'y a aucune autre embûche
                        If (CType(LaCarte, Kilometre)).Distance > 50 Then
                            AnnonceCarteInterdite(LaCarte, Interdictions.Limitation)
                            Return 'on ne peut pas dépasser 50
                        End If

                    Case Else 's'il y a n'importe quelle autre embuche, on ne peut pas rouler
                        AnnonceCarteInterdite(LaCarte, Interdictions.AttaqueEnCours)
                        Return
                End Select

                If (CType(LaCarte, Kilometre)).Distance = 200 AndAlso Jeu.OfType(Of Kilometre)().Count(Function(k) k.Distance = 200) = 2 Then
                    'On ne peut pas joueur plus de 2 "200 km"
                    'à noter pour cette recherche, au lieu d'imriquer des boucles (comme plus haut), j'ai utilisé une requête Linq, qui filtre sur le type puis compte le nombre de "200 km"
                    AnnonceCarteInterdite(LaCarte, Interdictions.Max200)
                    Return
                End If
            ElseIf TypeOf LaCarte Is Parade Then
                Dim maParade As Parade = CType(LaCarte, Parade)
                If Not EmbucheEnCours.HasFlag(maParade.EmbucheParee) Then
                    AnnonceCarteInterdite(LaCarte, Interdictions.ParadeIncoherente)
                    Return 'on ne peut pas parer une embuche que l'on ne subit pas
                End If

                If maParade.EmbucheParee = TypeEmbuche.FeuRouge AndAlso Not(EmbucheEnCours = TypeEmbuche.FeuRouge OrElse EmbucheEnCours = (TypeEmbuche.FeuRouge Or TypeEmbuche.LimitationVitesse)) Then
                    AnnonceCarteInterdite(LaCarte, Interdictions.ParadeAvantFeuVert)
                    Return 'on ne peut poser un feu vert qui si on est en "état" de feu rouge, ou feu rouge et limitation, toutes les autres embuches doivent avoir été parées avant
                End If

                EmbucheEnCours = EmbucheEnCours Xor maParade.EmbucheParee
            Else
                'c'est forcément une botte, elle peut se poser n'importe quand même sans parer une embûche
                Dim laBotte As Botte = (CType(LaCarte, Botte))

                If coupFourrePossible IsNot Nothing AndAlso coupFourrePossible Is Me Then 'le joueur doit avoir été attaqué, disposer de la botte et (réagir sous 10 s vérifié dans AutorisationDeJouer) et avant un autre joueur
                    FaitCoupFourre(laBotte)
                End If

                Immunite = Immunite Or laBotte.EmbucheImmunisee 'immunise

                If laBotte.EmbucheImmunisee.HasFlag(TypeEmbuche.FeuRouge) Then
                    If EmbucheEnCours.HasFlag(TypeEmbuche.FeuRouge) Then
                        EmbucheEnCours = EmbucheEnCours Xor TypeEmbuche.FeuRouge 'pare si besoin
                    End If

                    If EmbucheEnCours.HasFlag(TypeEmbuche.LimitationVitesse) Then
                        EmbucheEnCours = EmbucheEnCours Xor TypeEmbuche.LimitationVitesse 'pare si besoin
                    End If
                ElseIf EmbucheEnCours.HasFlag(laBotte.EmbucheImmunisee) Then
                    EmbucheEnCours = EmbucheEnCours Xor laBotte.EmbucheImmunisee 'pare si besoin
                End If
            End If

            chrono.Reset() 'Raz des variables
            coupFourrePossible = Nothing

            Main.Remove(LaCarte) 'On enlève la carte de la main

            Dim superieure As Carte = Jeu.FirstOrDefault(Function(c) c.Ordre > LaCarte.Ordre) 'on cherche la première carte dont l'ordre est supérieur à la carte jouée.
            If superieure IsNot Nothing Then
                Jeu.Insert(Jeu.IndexOf(superieure), LaCarte) 'si la carte supérieure existe, on insère la carte jouée à l'index de la carte supérieure
            Else
                Jeu.Add(LaCarte) 'sinon on ajoute la carte au jeu (donc à la fin)
            End If

            If TypeOf LaCarte Is Botte Then
                PiocheAutomatique() 'si c'est une botte, en coup fourré ou non, le joueur rejoue
            Else
                AnnonceFinTour() 'Annonce que le joueur à fini de joueur, ce qui entraine que le "tour" passe au suivant qui pioche automatiquement
            End If

        End Sub

        ''' <summary>
        ''' Méthode permettant de jouer un kilomètre
        ''' </summary>
        ''' <param name="Km"></param>
        ''' <remarks>Je crée 3 surcharges pour être sûr qu'une embuche ne sera pas transmise en paramètre</remarks>
        Public Sub Jouer(ByVal Km As Kilometre)
            Joue(Km)
        End Sub

        ''' <summary>
        ''' Surcharge permettant de jouer une parade
        ''' </summary>
        ''' <param name="LaParade"></param>
        Public Sub Jouer(ByVal LaParade As Parade)
            Joue(LaParade)
        End Sub

        ''' <summary>
        ''' Surcharge permettant de jouer une botte
        ''' </summary>
        ''' <param name="LaParade"></param>
        Public Sub Jouer(ByVal LaBotte As Botte)
            Joue(LaBotte)
        End Sub

        ''' <summary>
        ''' Surcharge permettant de mettre une embuche à un adversaire
        ''' </summary>
        ''' <param name="Lembuche"></param>
        ''' <param name="Cible"></param>
        Public Sub Jouer(ByVal Lembuche As Embuche, ByVal Cible As Joueur)
            If Not AutorisationDeJouer(Lembuche) Then
                Return
            End If

            If Cible Is Me Then 'si le joueur cible est le joueur en cours, on génère une erreur
                Throw New Exception("Le joueur ne peut pas se mettre une embuche à lui-même")
            End If

            If Cible.Immunite.HasFlag(Lembuche.Type) Then 'on ne peut pas poser ni une embûche contre laquelle la cible est immunisée,
                AnnonceCarteInterdite(Lembuche, Interdictions.Immunite)

            ElseIf Lembuche.Type = TypeEmbuche.LimitationVitesse AndAlso Cible.EmbucheEnCours.HasFlag(TypeEmbuche.LimitationVitesse) Then 'ni une limitation si elle y est déjà
                    AnnonceCarteInterdite(Lembuche, Interdictions.MaxLimitation)

            ElseIf Lembuche.Type <> TypeEmbuche.LimitationVitesse AndAlso Cible.EmbucheEnCours <> TypeEmbuche.Aucune AndAlso Cible.EmbucheEnCours <> TypeEmbuche.LimitationVitesse Then 'ni les autres embûches s'il y a en déjà une (sauf la limitation)
                AnnonceCarteInterdite(Lembuche, Interdictions.DejaAttaque)

            Else
                Main.Remove(Lembuche) 'On enlève la carte de la main

                Dim superieure As Carte = Cible.Jeu.FirstOrDefault(Function(c) c.Ordre > Lembuche.Ordre) 'on cherche la première carte dont l'ordre est supérieur à la carte jouée.
                If superieure IsNot Nothing Then
                    Cible.Jeu.Insert(Cible.Jeu.IndexOf(superieure), Lembuche) 'si la carte supérieure existe, on insère la carte jouée à l'index de la carte supérieure
                Else
                    Cible.Jeu.Add(Lembuche) 'sinon on ajoute la carte au jeu (donc à la fin)
                End If

                AnnonceFinTour()
            End If
        End Sub

        ''' <summary>
        ''' Génère l'évènement
        ''' </summary>
        Private Sub AnnonceCarteInterdite(ByVal LaCarte As Carte, ByVal Raison As Interdictions)
            RaiseEvent CarteInterdite(Me, LaCarte, Raison)
        End Sub

        ''' <summary>
        ''' Pioche de façon automatique si le joueur a un niveau "bas"
        ''' </summary>
        Public Sub PiocheAutomatique()
            If Niveau = Difficulte.Facile Then
                Pioche()
            End If
        End Sub

        ''' <summary>
        ''' Pioche une carte
        ''' </summary>
        Public Sub Pioche()
            If Moteur.PartieEnCours AndAlso Moteur.JoueurEnCours IsNot Me Then
                AnnonceCarteInterdite(Nothing, Interdictions.PasSonTour)
                Return
            End If


            Dim cPiochee As Carte = Moteur.Pioche(0)
            Moteur.Pioche.Remove(cPiochee) 'pioche la première carte

            If Main.Count > 6 Then
                AnnonceCarteInterdite(cPiochee, Interdictions.PiocheInterdite)
                Return
            End If

            Dim superieure As Carte = Main.FirstOrDefault(Function(c) c.Ordre > cPiochee.Ordre) 'on cherche la première carte dont l'ordre est supérieur à la carte jouée.
            If superieure IsNot Nothing Then
                Main.Insert(Main.IndexOf(superieure), cPiochee) 'si la carte supérieure existe, on insère la carte jouée à l'index de la carte supérieure
            Else
                Main.Add(cPiochee) 'sinon on ajoute la carte au jeu (donc à la fin)
            End If
        End Sub

        ''' <summary>
        ''' Teste si le joueur a le droit de jouer
        ''' </summary>
        ''' <param name="LaCarte"></param>
        ''' <returns></returns>
        Private Function AutorisationDeJouer(ByVal LaCarte As Carte) As Boolean
            If coupFourrePossible IsNot Nothing AndAlso chrono.ElapsedMilliseconds < 11e3 Then
                If coupFourrePossible IsNot Me AndAlso coupFourrePossible.Niveau = Difficulte.Normale Then 'si un joueur de niveau Normal a la possibilité de joueur un coup fourré on lui laisse 10s
                    AnnonceCarteInterdite(LaCarte, Interdictions.Patienter)
                    Return False
                End If
                If coupFourrePossible Is Me AndAlso TypeOf LaCarte Is Botte AndAlso (CType(coupFourrePossible.DerniereCarte, Embuche)).Type.HasFlag((CType(LaCarte, Botte)).EmbucheImmunisee) Then 'si un joueur joue son coup fourré
                    Return True
                End If
            End If
            If Moteur.JoueurEnCours IsNot Me Then
                 AnnonceCarteInterdite(LaCarte, Interdictions.PasSonTour)
                 Return False
            End If
            If Main.Count < 7 Then
                RaiseEvent PiocherDabord(Me)
                Return False
            End If

            Return True
        End Function

        ''' <summary>
        ''' Défausse une carte
        ''' </summary>
        ''' <param name="LaCarte">Carte à défausser</param>
        ''' <param name="PasseLeTour">Indique si le joueur doit passer son tour</param>
        Public Sub Defausse(ByVal LaCarte As Carte, Optional ByVal PasseLeTour As Boolean = True)
            If Not AutorisationDeJouer(LaCarte) Then
                Return
            End If

            Moteur.Defausse.Add(LaCarte)
            Main.Remove(LaCarte)
            If PasseLeTour Then
                AnnonceFinTour()
            End If

        End Sub

        ''' <summary>
        ''' Annonce que le tour est terminé
        ''' </summary>
        Private Sub AnnonceFinTour()
            RaiseEvent FiniMonTour(Me)
        End Sub

        ''' <summary>
        ''' Réécriture de ToString qui affiche le nom
        ''' </summary>
        ''' <returns></returns>
        Public Overrides Function ToString() As String
            Return Nom
        End Function
        #End Region

        #Region "INotifyPropertyChanged"
        Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged
        ''' <summary>
        ''' Vérifie s'il y a un abonné à l'évènenemet et le génère
        ''' </summary>
        ''' <param name="Propriete"></param>
        Private Sub RaisePropertyChanged(ByVal Propriete As String)
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(Propriete))
        End Sub

        #End Region

    End Class
End Namespace
