﻿Imports System.Collections.ObjectModel
Imports System.Text
Imports System.Threading.Tasks

Namespace JeuAvecDesKilometres
    Public NotInheritable Class Moteur
        Public Shared Event CarteInterdite As EvenementCarte 'évènement signalant que le joueur tente de poser une carte à l'encontre des règles.

        Public Delegate Sub EvenementJoueur(ByVal LeJoueur As Joueur) 'signature de l'évènement
        Public Shared Event ProchainJoueur As EvenementJoueur 'évènement annonçant le joueur suivant
        Public Shared Event PartieGagnee As EvenementJoueur 'évènement annonçant qui a gagné
        Public Shared Event JoueurDisqualifie As EvenementJoueur 'évènement annonçant qui est disqualifié
        Public Shared Event PiocheDabord As EvenementJoueur 'évènement annonçant que le joueur doit piocher d'abord

        Public Delegate Sub EvenementMoteurJeu() 'signature de l'évènement
        Public Shared Event NouvellePioche As EvenementMoteurJeu 'évènement annonçant la nouvelle pioche
        Public Shared Event CoupFourre As EvenementMoteurJeu 'évènement annonçant un coup fourré

        ''' <summary>
        ''' Distance à parcourir pour terminer une partie
        ''' </summary>
        Private Shared privateDistanceAParcourir As Integer
        Public Shared Property DistanceAParcourir As Integer
            Get
                Return privateDistanceAParcourir
            End Get
            Private Set(ByVal value As Integer)
                privateDistanceAParcourir = value
            End Set
        End Property

        ''' <summary>
        ''' Liste des joueurs
        ''' </summary>
        Private Shared privateJoueurs As List(Of Joueur)
        Public Shared Property Joueurs As List(Of Joueur)
            Get
                Return privateJoueurs
            End Get
            Private Set(ByVal value As List(Of Joueur))
                privateJoueurs = value
            End Set
        End Property

        ''' <summary>
        ''' Collection de cartes défaussées
        ''' </summary>
        Public Shared Property Defausse As ObservableCollection(Of Carte)

        Public Shared Property Pioche As ObservableCollection(Of Carte)
        ''' <summary>
        ''' Collection de cartes à piocher
        ''' </summary>

        ''' <summary>
        ''' Joueur dont c'est le tour de jouer
        ''' </summary>
        Private Sub New()
        End Sub
        Public Shared ReadOnly Property JoueurEnCours As Joueur
            Get
                Return leJoueurEnCours
            End Get
        End Property

        Private Shared leJoueurEnCours As Joueur

        ''' <summary>
        ''' Indique si une partie est en cours
        ''' </summary>
        Private Shared privatePartieEnCours As Boolean
        Public Shared Property PartieEnCours As Boolean
            Get
                Return privatePartieEnCours
            End Get
            Private Set(ByVal value As Boolean)
                privatePartieEnCours = value
            End Set
        End Property

        ''' <summary>
        ''' Initialise les éléments de jeu avant la première partie
        ''' </summary>
        ''' <param name="NomsJoueurs">Liste avec les noms des joueurs</param>
        Public Shared Sub InitPremierePartie(ByVal NomsJoueurs As List(Of String))
            Dim lesJoueurs As New Dictionary(Of String, Difficulte)()
            For Each joueur As String In NomsJoueurs
                lesJoueurs.Add(joueur,Difficulte.Facile)
            Next joueur

            InitPremierePartie(lesJoueurs)
        End Sub

        ''' <summary>
        ''' Initialise les éléments de jeu avant la première partie
        ''' </summary>
        ''' <param name="LesJoueurs">Dictionnaire avec les noms des joueurs et leurs niveau</param>
        Public Shared Sub InitPremierePartie(ByVal LesJoueurs As Dictionary(Of String, Difficulte))
            Defausse = New ObservableCollection(Of Carte)()
            Pioche = New ObservableCollection(Of Carte)()
            AddHandler Pioche.CollectionChanged, AddressOf Pioche_CollectionChanged
            Joueurs = New List(Of Joueur)()

            'Crée les joueurs et s'abonne aux évènements
            For Each joueur In LesJoueurs
                Dim j As New Joueur(joueur.Key,joueur.Value)
                AddHandler j.CoupFourre, AddressOf j_CoupFourre
                AddHandler j.DistanceDepassee, AddressOf j_DistanceDepassee
                AddHandler j.JaiGagne, AddressOf j_JaiGagne
                AddHandler j.FiniMonTour, AddressOf j_FiniMonTour
                AddHandler j.CarteInterdite, AddressOf j_CarteInterdite
                AddHandler j.PiocherDabord, AddressOf j_PiocherDabord

                Joueurs.Add(j)
            Next joueur


            DistanceAParcourir = If(Joueurs.Count > 3, 700, 1000) 'La distance à parcourir est 700 à partir de 4 joueurs, sinon 1000

            NouvellePartie()

        End Sub

        ''' <summary>
        ''' Annonce que le joueur doit piocher avant de joueur
        ''' </summary>
        ''' <param name="LeJoueur"></param>
        Private Shared Sub j_PiocherDabord(ByVal LeJoueur As Joueur)
            RaiseEvent PiocheDabord(LeJoueur)

        End Sub

        ''' <summary>
        ''' Transmet l'alerte qu'un joueur tente de joueur une carte "illégale"
        ''' </summary>
        ''' <param name="LeJoueur"></param>
        ''' <param name="LaCarte"></param>
        Private Shared Sub j_CarteInterdite(ByVal LeJoueur As Joueur, ByVal LaCarte As Carte, ByVal Raison As Interdictions)
            If Raison <> Interdictions.Patienter Then 'Patienter est dans le cas ou un joueur "A" de niveau "Normal" peut jouer un coup fourré et qu'un autre joueur "B" joue avant les 10 secondes, le joueur "B" ne doit pas être pénalisé

                Select Case LeJoueur.Niveau 'selon le niveau du joueur, on défausse la carte, ce qui passe au joueur suivant
                    Case Difficulte.Difficile, Difficulte.Expert
                        Raison = Interdictions.Inconnue 'à ces niveaux, la raison n'est plus affichée.
                        LeJoueur.Defausse(LaCarte, Raison <> Interdictions.PasSonTour)

                    Case Else
                        If Raison = Interdictions.PiocheInterdite Then
                            LeJoueur.Defausse(LaCarte, False)
                        End If

                End Select
            End If

            RaiseEvent CarteInterdite(LeJoueur, LaCarte, Raison)
        End Sub

        ''' <summary>
        ''' Quand un joueur à finit, on passe au suivant
        ''' </summary>
        ''' <param name="LeJoueur"></param>
        Private Shared Sub j_FiniMonTour(ByVal LeJoueur As Joueur)
            JoueurSuivant(LeJoueur)
        End Sub

        ''' <summary>
        ''' Signale la fin de la partie
        ''' </summary>
        ''' <param name="LeJoueur"></param>
        ''' <param name="Kilometres"></param>
        Private Shared Sub j_JaiGagne(ByVal LeJoueur As Joueur, ByVal Kilometres As Integer)
            PartieEnCours = False

            For Each j As Joueur In Joueurs
                j.FinPartie(numeroPartie)
            Next j

            RaiseEvent PartieGagnee(LeJoueur)

        End Sub


        Private Shared numeroPartie As Integer = 0

        ''' <summary>
        ''' Démarre une nouvelle partie
        ''' </summary>
        Public Shared Sub NouvellePartie()
            numeroPartie += 1

            'génération des cartes
            Dim toutesLesCartes As List(Of Carte) = Kilometre.Generer()
            toutesLesCartes.AddRange(Embuche.Generer())
            toutesLesCartes.AddRange(Parade.Generer())
            toutesLesCartes.AddRange(Botte.Generer())

            'Efface les cartes présentes dans le jeu
            Pioche.Clear()
            Defausse.Clear()
            For Each j As Joueur In Joueurs
                j.NouvellePartie()
            Next j

            'mélange des cartes
            Melanger(toutesLesCartes)

            'distribution des cartes 6 par joueur, une par une chacun son tour
            For i As Integer = 0 To 5
                For j As Integer = 0 To Moteur.Joueurs.Count - 1
                    Moteur.Joueurs(j).Pioche()
                Next j
            Next i

            leJoueurEnCours = Joueurs(0)
            AnnonceProchainJoueur()

            PartieEnCours = True

        End Sub

        Private Shared indexJoueursDisqualifies As New List(Of Integer)()
        ''' <summary>
        ''' Un joueur a dépassé la distance=> il est éliminé
        ''' </summary>
        ''' <param name="LeJoueur"></param>
        ''' <param name="Kilometres"></param>
        Private Shared Sub j_DistanceDepassee(ByVal LeJoueur As Joueur, ByVal Kilometres As Integer)
            If LeJoueur.Niveau = Difficulte.Expert Then 'si le joueur est expert, cette erreur le disqualifie
                indexJoueursDisqualifies.Add(Joueurs.IndexOf(LeJoueur))
                RaiseEvent JoueurDisqualifie(LeJoueur)
            End If
        End Sub


        ''' <summary>
        ''' Change le joueur après un coup fourré
        ''' </summary>
        ''' <param name="LeJoueur"></param>
        Private Shared Sub j_CoupFourre(ByVal LeJoueur As Joueur)
            leJoueurEnCours = LeJoueur
            AnnonceProchainJoueur()

            RaiseEvent CoupFourre()
        End Sub

        ''' <summary>
        ''' Annonce le joueur suivant
        ''' </summary>
        Private Shared Sub AnnonceProchainJoueur()

            For Each j As Joueur In Joueurs.Where(Function(jj) jj.Actif)
                j.Actif = False '"desactive" l'ancien joueur
            Next j

            JoueurEnCours.Actif = True '"active" le joueur en cours

            JoueurEnCours.PiocheAutomatique() 'fait piocher le joueur suivant
            RaiseEvent ProchainJoueur(JoueurEnCours)
        End Sub

        ''' <summary>
        ''' Surveille la pioche, et la refait à partir de la défausse quand elle est vide
        ''' </summary>
        ''' <param name="sender"></param>
        ''' <param name="e"></param>
        Private Shared Sub Pioche_CollectionChanged(ByVal sender As Object, ByVal e As System.Collections.Specialized.NotifyCollectionChangedEventArgs)
            If e.Action <> System.Collections.Specialized.NotifyCollectionChangedAction.Remove Then
                Return
            End If

            If Pioche.Count = 0 Then
                Melanger(Defausse.ToList())
                RaiseEvent NouvellePioche()
            End If
        End Sub

        ''' <summary>
        ''' Mélange les cartes et (re)crée la pioche
        ''' </summary>
        ''' <param name="Cartes"></param>
        Private Shared Sub Melanger(ByVal Cartes As List(Of Carte))
            Dim rnd As New Random()
            Do
                Dim index As Integer = rnd.Next(Cartes.Count) 'génère un entier aléatoire compris entre 0 et Cartes.Count -1
                Pioche.Add(Cartes(index)) 'ajoute la carte à cet index dans la pioche
                Cartes.RemoveAt(index) 'enlève la carte de la collection d'origine
            Loop While Cartes.Count > 0
        End Sub

        ''' <summary>
        ''' Passe au joueur suivant
        ''' </summary>
        Private Shared Sub JoueurSuivant(ByVal LeJoueur As Joueur)
            Dim indexJoueur As Integer = Joueurs.IndexOf(LeJoueur)

            If indexJoueur = Joueurs.Count - 1 Then
                indexJoueur = 0
            Else
                indexJoueur += 1
            End If

            leJoueurEnCours = Joueurs(indexJoueur)

            If indexJoueursDisqualifies.Contains(indexJoueur) Then 'si le joueur est disqualifié on passe au suivant de manière récursive
                JoueurSuivant(LeJoueur)

            Else
                AnnonceProchainJoueur()
            End If
        End Sub
    End Class

End Namespace
