﻿Imports JeuAvecDesKilometres
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows.Media.Animation

Namespace Tuto_Objets_Csharp_WPF
    ''' <summary>
    ''' Logique d'interaction pour MainWindow.xaml
    ''' </summary>
    Partial Public Class MainWindow
        Inherits Window
        Public Sub New()
            InitializeComponent()
        End Sub

        Private Sub Window_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
            Dim nomsJoueurs As New Dictionary(Of String, Difficulte)()
            nomsJoueurs.Add("Joueur A", Difficulte.Normale)
            nomsJoueurs.Add("Joueur B", Difficulte.Normale)
            nomsJoueurs.Add("Joueur C", Difficulte.Normale)

            Moteur.InitPremierePartie(nomsJoueurs)

            lesJoueurs.DataContext = Moteur.Joueurs 'binde tous les joueurs
            lblPioche.DataContext = Moteur.Pioche.Count 'binde le nombre de cartes dans la pioche
            lstDefausse.DataContext = Moteur.Defausse 'binde la defausse

            AddHandler Moteur.PartieGagnee, AddressOf Moteur_PartieGagnee
            AddHandler Moteur.CarteInterdite, AddressOf Moteur_CarteInterdite
            AddHandler Moteur.CoupFourre, AddressOf Moteur_CoupFourre
            AddHandler Moteur.JoueurDisqualifie, AddressOf Moteur_JoueurDisqualifie
            AddHandler Moteur.PiocheDabord, AddressOf Moteur_PiocheDabord


        End Sub

        Private Sub butPiocher_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            Dim j As Joueur = TrouveJoueur(sender)
            j.Pioche()
        End Sub

        Private Sub butJouer_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            Dim j As Joueur = TrouveJoueur(sender)
            Dim laCarte As Carte = TrouveCarte(sender)

            If laCarte Is Nothing Then
                Return
            End If

            If TypeOf laCarte Is Botte Then
                j.Jouer(CType(laCarte, Botte))
            ElseIf TypeOf laCarte Is Kilometre Then
                j.Jouer(CType(laCarte, Kilometre))
            ElseIf TypeOf laCarte Is Parade Then
                j.Jouer(CType(laCarte, Parade))
            Else
                attaquant = j
                attaque = CType(laCarte, Embuche)
                AfficheMessage("Cliquer sur la cible")
            End If
        End Sub
        Private attaquant As Joueur
        Private attaque As Embuche

        Private Sub butDefausser_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            Dim j As Joueur = TrouveJoueur(sender)
            Dim laCarte As Carte = TrouveCarte(sender)
            If laCarte Is Nothing Then
                Return
            End If

            j.Defausse(laCarte)
        End Sub


        Private Function TrouveJoueur(ByVal sender As Object) As Joueur
            'On peut remonter à l'instance qui a bindé le bouton clické
            Dim bouton As Button = CType(sender, Button)
            Dim instance As Object = bouton.DataContext
            Return CType(instance, Joueur)
        End Function

        Private Function TrouveCarte(ByVal sender As Object) As Carte
            'on peut aussi remonter à la carte sélectionnée dans la liste
            Dim bouton As Button = CType(sender, Button)
            Dim parent1 As StackPanel = CType(bouton.Parent, StackPanel)

            Dim parent2 As StackPanel
            If parent1.Name = "splJoueur" Then
                parent2 = parent1
            Else
                parent2 = CType(parent1.Parent, StackPanel)
            End If

            Dim main As ListBox = parent2.Children.OfType(Of ListBox)().Single(Function(lb) lb.Name = "Main")
            Return CType(main.SelectedItem, Carte)

        End Function

        Private Sub lesJoueurs_MouseUp(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            If attaquant IsNot Nothing AndAlso lesJoueurs.SelectedItem IsNot attaquant Then
                attaquant.Jouer(attaque, CType(lesJoueurs.SelectedItem, Joueur))
                attaque = Nothing
                attaquant = Nothing
            End If
            lesJoueurs.SelectedIndex = -1
        End Sub


        Private Sub Moteur_PiocheDabord(ByVal LeJoueur As Joueur)
            AfficheMessage("Le joueur doit piocher")

        End Sub

        Private Sub AfficheMessage(ByVal Texte As String)
            Dim animation As New DoubleAnimation(1, 0, TimeSpan.FromSeconds(3))
            lblMessage.BeginAnimation(Rectangle.OpacityProperty, animation)

            lblMessage.Content = Texte

        End Sub

        Private Sub Moteur_JoueurDisqualifie(ByVal LeJoueur As Joueur)
            AfficheMessage(LeJoueur.Nom & " est disqualifié")
        End Sub

        Private Sub Moteur_CoupFourre()
            AfficheMessage("Coup fourré!")
        End Sub

        Private Sub Moteur_CarteInterdite(ByVal LeJoueur As Joueur, ByVal LaCarte As Carte, ByVal Raison As Interdictions)
            Dim messages() As String = {"Joueur immunisé","Limitation de vitesse","Embûche à parer","2 cartes 200 km maximum","Parade inutile", "Parez l'embûche avant de poser le feu vert","La cible est déjà attaquée","Joueur déjà bridé", "Carte interdite pour une raison inconnue","Distance dépassée","Veuillez patienter","Ce n'est pas votre tour","Pioche interdite"}
            AfficheMessage(messages(CInt(Fix(Raison))))
        End Sub

        Private Sub Moteur_PartieGagnee(ByVal LeJoueur As Joueur)
            Dim lautre As Joueur = Moteur.Joueurs.Single(Function(j) j IsNot LeJoueur)
            Dim str As New StringBuilder(LeJoueur.Nom)
            str.AppendLine(" a gagné!")
            str.AppendLine(String.Format("{0} marque {1} points.", LeJoueur.Nom, LeJoueur.Points))
            str.AppendLine(String.Format("{0} marque {1} points.", lautre.Nom, lautre.Points))


            MessageBox.Show(str.ToString())

        End Sub
    End Class
End Namespace
